@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2005 Nominum, Inc.
+# Copyright (C) 2001-2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -35,7 +35,7 @@ class Renderer(object):
     class and its to_wire() method to generate wire-format messages.
     This class is for those applications which need finer control
     over the generation of messages.
-    
+
     Typical use::
 
         r = dns.renderer.Renderer(id=1, flags=0x80, max_size=512)
@@ -85,7 +85,7 @@ class Renderer(object):
         @param origin: the origin to use when rendering relative names
         @type origin: dns.name.Namem or None.
         """
-        
+
         self.output = cStringIO.StringIO()
         if id is None:
             self.id = random.randint(0, 65535)
@@ -108,7 +108,7 @@ class Renderer(object):
         @param where: the offset
         @type where: int
         """
-        
+
         self.output.seek(where)
         self.output.truncate()
         keys_to_delete = []
@@ -129,7 +129,7 @@ class Renderer(object):
         @raises dns.exception.FormError: an attempt was made to set
         a section value less than the current section.
         """
-        
+
         if self.section != section:
             if self.section > section:
                 raise dns.exception.FormError
@@ -145,7 +145,7 @@ class Renderer(object):
         @param rdclass: the question rdata class
         @type rdclass: int
         """
-        
+
         self._set_section(QUESTION)
         before = self.output.tell()
         qname.to_wire(self.output, self.compress, self.origin)
@@ -155,13 +155,13 @@ class Renderer(object):
             self._rollback(before)
             raise dns.exception.TooBig
         self.counts[QUESTION] += 1
-        
+
     def add_rrset(self, section, rrset, **kw):
         """Add the rrset to the specified section.
 
         Any keyword arguments are passed on to the rdataset's to_wire()
         routine.
-        
+
         @param section: the section
         @type section: int
         @param rrset: the rrset
@@ -191,7 +191,7 @@ class Renderer(object):
         @param rdataset: the rdataset
         @type rdataset: dns.rdataset.Rdataset object
         """
-        
+
         self._set_section(section)
         before = self.output.tell()
         n = rdataset.to_wire(name, self.output, self.compress, self.origin,
@@ -202,7 +202,7 @@ class Renderer(object):
             raise dns.exception.TooBig
         self.counts[section] += n
 
-    def add_edns(self, edns, ednsflags, payload):
+    def add_edns(self, edns, ednsflags, payload, options=None):
         """Add an EDNS OPT record to the message.
 
         @param edns: The EDNS level to use.
@@ -212,13 +212,37 @@ class Renderer(object):
         @param payload: The EDNS sender's payload field, which is the maximum
         size of UDP datagram the sender can handle.
         @type payload: int
+        @param options: The EDNS options list
+        @type options: list of dns.edns.Option instances
         @see: RFC 2671
         """
 
+        # make sure the EDNS version in ednsflags agrees with edns
+        ednsflags &= 0xFF00FFFFL
+        ednsflags |= (edns << 16)
         self._set_section(ADDITIONAL)
         before = self.output.tell()
         self.output.write(struct.pack('!BHHIH', 0, dns.rdatatype.OPT, payload,
                                       ednsflags, 0))
+        if not options is None:
+            lstart = self.output.tell()
+            for opt in options:
+                stuff = struct.pack("!HH", opt.otype, 0)
+                self.output.write(stuff)
+                start = self.output.tell()
+                opt.to_wire(self.output)
+                end = self.output.tell()
+                assert end - start < 65536
+                self.output.seek(start - 2)
+                stuff = struct.pack("!H", end - start)
+                self.output.write(stuff)
+                self.output.seek(0, 2)
+            lend = self.output.tell()
+            assert lend - lstart < 65536
+            self.output.seek(lstart - 2)
+            stuff = struct.pack("!H", lend - lstart)
+            self.output.write(stuff)
+            self.output.seek(0, 2)
         after = self.output.tell()
         if after >= self.max_size:
             self._rollback(before)
@@ -226,14 +250,14 @@ class Renderer(object):
         self.counts[ADDITIONAL] += 1
 
     def add_tsig(self, keyname, secret, fudge, id, tsig_error, other_data,
-                 request_mac):
+                 request_mac, algorithm=dns.tsig.default_algorithm):
         """Add a TSIG signature to the message.
-        
+
         @param keyname: the TSIG key name
         @type keyname: dns.name.Name object
         @param secret: the secret to use
         @type secret: string
-        @param fudge: TSIG time fudge; default is 300 seconds.
+        @param fudge: TSIG time fudge
         @type fudge: int
         @param id: the message id to encode in the tsig signature
         @type id: int
@@ -244,20 +268,23 @@ class Renderer(object):
         @param request_mac: This message is a response to the request which
         had the specified MAC.
         @type request_mac: string
+        @param algorithm: the TSIG algorithm to use
+        @type algorithm: dns.name.Name object
         """
- 
+
         self._set_section(ADDITIONAL)
         before = self.output.tell()
         s = self.output.getvalue()
-        (tsig_rdata, self.mac, ctx) = dns.tsig.hmac_md5(s,
-                                                        keyname,
-                                                        secret,
-                                                        int(time.time()),
-                                                        fudge,
-                                                        id,
-                                                        tsig_error,
-                                                        other_data,
-                                                        request_mac)
+        (tsig_rdata, self.mac, ctx) = dns.tsig.sign(s,
+                                                    keyname,
+                                                    secret,
+                                                    int(time.time()),
+                                                    fudge,
+                                                    id,
+                                                    tsig_error,
+                                                    other_data,
+                                                    request_mac,
+                                                    algorithm=algorithm)
         keyname.to_wire(self.output, self.compress, self.origin)
         self.output.write(struct.pack('!HHIH', dns.rdatatype.TSIG,
                                       dns.rdataclass.ANY, 0, 0))
@@ -278,11 +305,11 @@ class Renderer(object):
     def write_header(self):
         """Write the DNS message header.
 
-        Writing the DNS message header is done asfter all sections
+        Writing the DNS message header is done after all sections
         have been rendered, but before the optional TSIG signature
         is added.
         """
-        
+
         self.output.seek(0)
         self.output.write(struct.pack('!HHHHHH', self.id, self.flags,
                                       self.counts[0], self.counts[1],
@@ -294,5 +321,5 @@ class Renderer(object):
 
         @rtype: string
         """
-        
+
         return self.output.getvalue()
