@@ -1,4 +1,4 @@
-# Copyright (C) 2003, 2004 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -38,7 +38,7 @@ def to_text(keyring):
     
     textring = {}
     for keyname in keyring:
-        keytext = dns.name.to_text(keyname)
+        keytext = keyname.to_text()
         secret = base64.encodestring(keyring[keyname])
         textring[keytext] = secret
     return textring
